# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Type, Union, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.radar.annotations import outage_get_params, outage_locations_params
from ....types.radar.annotations.outage_get_response import OutageGetResponse
from ....types.radar.annotations.outage_locations_response import OutageLocationsResponse

__all__ = ["OutagesResource", "AsyncOutagesResource"]


class OutagesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> OutagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return OutagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OutagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return OutagesResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        asn: int | Omit = omit,
        bot: str | Omit = omit,
        ca: str | Omit = omit,
        data_source: Literal[
            "ALL",
            "AI_BOTS",
            "AI_GATEWAY",
            "BGP",
            "BOTS",
            "CONNECTION_ANOMALY",
            "CT",
            "DNS",
            "DNS_MAGNITUDE",
            "DNS_AS112",
            "DOS",
            "EMAIL_ROUTING",
            "EMAIL_SECURITY",
            "FW",
            "FW_PG",
            "HTTP",
            "HTTP_CONTROL",
            "HTTP_CRAWLER_REFERER",
            "HTTP_ORIGINS",
            "IQI",
            "LEAKED_CREDENTIALS",
            "NET",
            "ROBOTS_TXT",
            "SPEED",
            "WORKERS_AI",
        ]
        | Omit = omit,
        date_end: Union[str, datetime] | Omit = omit,
        date_range: str | Omit = omit,
        date_start: Union[str, datetime] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        geo_id: str | Omit = omit,
        limit: int | Omit = omit,
        location: str | Omit = omit,
        log: str | Omit = omit,
        offset: int | Omit = omit,
        origin: str | Omit = omit,
        outage_cause: Literal[
            "BLOCKING",
            "CABLE_CUT",
            "CYBERATTACK",
            "DNS",
            "FIRE",
            "GOVERNMENT_DIRECTED",
            "MAINTENANCE",
            "MECHANICAL",
            "MILITARY_ACTION",
            "MISCONFIGURATION",
            "NATURAL_DISASTER",
            "NETWORK_PROBLEM",
            "POWER_OUTAGE",
            "SOFTWARE",
            "TECHNICAL_PROBLEM",
            "UNKNOWN",
            "WEATHER",
        ]
        | Omit = omit,
        outage_type: Literal["NATIONWIDE", "REGIONAL", "NETWORK", "PLATFORM"] | Omit = omit,
        query: str | Omit = omit,
        tags: List[
            Literal[
                "ADM1",
                "ADM2",
                "API_TRAFFIC",
                "ARC",
                "AS",
                "ASN",
                "ATTACKS",
                "AUTHOR",
                "BANDWIDTH",
                "BITRATE",
                "BOT",
                "BOT_CATEGORY",
                "BOT_CLASS",
                "BOT_KIND",
                "BOT_OPERATOR",
                "BROWSER",
                "BROWSER_FAMILY",
                "BYTES",
                "CA",
                "CACHE_HIT",
                "CA_OWNER",
                "CHECK_RESULT",
                "CLIENT_TYPE",
                "COMPROMISED",
                "CONTENT_TYPE",
                "CRAWL_PURPOSE",
                "CRAWL_REFER_RATIO",
                "DEVICE_TYPE",
                "DKIM",
                "DMARC",
                "DNS",
                "DNSSEC",
                "DNSSEC_AWARE",
                "DNSSEC_E2E",
                "DOMAIN_CATEGORY",
                "DURATION",
                "EDNS",
                "ENCRYPTED",
                "ENTRY_TYPE",
                "EXPIRATION_STATUS",
                "HAS_IPS",
                "HAS_MATCHING_ANSWER",
                "HAS_WILDCARDS",
                "HTTP_METHOD",
                "HTTP_PROTOCOL",
                "HTTP_VERSION",
                "INDUSTRY",
                "IP_VERSION",
                "JITTER",
                "KEY_AGREEMENT",
                "LATENCY",
                "LOCATION",
                "LOCATION_LATENCY",
                "LOG",
                "LOG_API",
                "LOG_OPERATOR",
                "MALICIOUS",
                "MANAGED_RULES",
                "MITIGATION_PRODUCT",
                "MODEL",
                "NAMESERVER_LATENCY",
                "ORIGIN",
                "ORIGIN_AS",
                "ORIGIN_LOCATION",
                "ORIGIN_TARGET_LOCATION_PAIR",
                "OS",
                "PERCENTILE",
                "POST_QUANTUM",
                "PREFIX",
                "PRODUCT",
                "PROTOCOL",
                "PROVIDER",
                "PUBLIC_KEY_ALGORITHM",
                "QUERY_TYPE",
                "REFERER",
                "REGION",
                "RESPONSE_CODE",
                "RESPONSE_STATUS",
                "RESPONSE_STATUS_CATEGORY",
                "RESPONSE_TTL",
                "SIGNATURE_ALGORITHM",
                "SPAM",
                "SPF",
                "SPOOF",
                "SUCCESS_RATE",
                "TARGET_LOCATION",
                "TASK",
                "THREAT_CATEGORY",
                "TLD",
                "TLD_DNS_MAGNITUDE",
                "TLS_VERSION",
                "UPDATE_TYPE",
                "USER_AGENT",
                "VALIDATION_LEVEL",
                "VECTOR",
                "VERTICAL",
            ]
        ]
        | Omit = omit,
        tld: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OutageGetResponse:
        """
        Retrieves the latest Internet outages and anomalies.

        Args:
          asn: Filters results by Autonomous System. Specify a single Autonomous System Number
              (ASN) as integer.

          bot: Filters results by bot.

          ca: Filters results by certificate authority.

          data_source: Filters results by data source.

          date_end: End of the date range (inclusive). Alternative to `dateRange`; provide together
              with `dateStart`.

          date_range: Filters results by a relative date range ending at the current time. Use `<n>d`
              for days (up to `364d`) or `<n>w` for weeks (up to `52w`), e.g. `7d`. Append
              `control` to request the equivalent previous period for comparison: the
              comparison window is shifted back by the current window's length rounded up to a
              whole number of weeks, so it keeps the same weekday alignment and does not
              overlap the current window (e.g. `3dcontrol` covers days -10 to -7, `7dcontrol`
              covers days -14 to -7, `28dcontrol` covers days -56 to -28, and `10dcontrol`
              covers days -24 to -14). Mutually exclusive with `dateStart`/`dateEnd`.

          date_start: Start of the date range (inclusive). Alternative to `dateRange`; provide
              together with `dateEnd`.

          format: Format in which results will be returned.

          geo_id: Filters results by geolocation. Refer to
              [GeoNames](https://download.geonames.org/export/dump/readme.txt).

          limit: Limits the number of objects returned in the response.

          location: Filters results by location. Specify an alpha-2 location code.

          log: Filters results by certificate log.

          offset: Skips the specified number of objects before fetching the results.

          origin: Filters results by origin.

          outage_cause: Filters results by outage cause.

          outage_type: Filters results by outage type.

          query: Filters results by a free-text match on the annotation description, id, or
              linked entities (location, ASN, origin).

          tags: Filters results by annotation tag. Matches annotations carrying at least one of
              the given tags.

          tld: Filters results by top-level domain.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/annotations/outages",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "asn": asn,
                        "bot": bot,
                        "ca": ca,
                        "data_source": data_source,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "geo_id": geo_id,
                        "limit": limit,
                        "location": location,
                        "log": log,
                        "offset": offset,
                        "origin": origin,
                        "outage_cause": outage_cause,
                        "outage_type": outage_type,
                        "query": query,
                        "tags": tags,
                        "tld": tld,
                    },
                    outage_get_params.OutageGetParams,
                ),
                post_parser=ResultWrapper[OutageGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[OutageGetResponse], ResultWrapper[OutageGetResponse]),
        )

    def locations(
        self,
        *,
        date_end: Union[str, datetime] | Omit = omit,
        date_range: str | Omit = omit,
        date_start: Union[str, datetime] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OutageLocationsResponse:
        """
        Retrieves the number of outages by location.

        Args:
          date_end: End of the date range (inclusive). Alternative to `dateRange`; provide together
              with `dateStart`.

          date_range: Filters results by a relative date range ending at the current time. Use `<n>d`
              for days (up to `364d`) or `<n>w` for weeks (up to `52w`), e.g. `7d`. Append
              `control` to request the equivalent previous period for comparison: the
              comparison window is shifted back by the current window's length rounded up to a
              whole number of weeks, so it keeps the same weekday alignment and does not
              overlap the current window (e.g. `3dcontrol` covers days -10 to -7, `7dcontrol`
              covers days -14 to -7, `28dcontrol` covers days -56 to -28, and `10dcontrol`
              covers days -24 to -14). Mutually exclusive with `dateStart`/`dateEnd`.

          date_start: Start of the date range (inclusive). Alternative to `dateRange`; provide
              together with `dateEnd`.

          format: Format in which results will be returned.

          limit: Limits the number of objects returned in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/annotations/outages/locations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit": limit,
                    },
                    outage_locations_params.OutageLocationsParams,
                ),
                post_parser=ResultWrapper[OutageLocationsResponse]._unwrapper,
            ),
            cast_to=cast(Type[OutageLocationsResponse], ResultWrapper[OutageLocationsResponse]),
        )


class AsyncOutagesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncOutagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOutagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOutagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncOutagesResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        asn: int | Omit = omit,
        bot: str | Omit = omit,
        ca: str | Omit = omit,
        data_source: Literal[
            "ALL",
            "AI_BOTS",
            "AI_GATEWAY",
            "BGP",
            "BOTS",
            "CONNECTION_ANOMALY",
            "CT",
            "DNS",
            "DNS_MAGNITUDE",
            "DNS_AS112",
            "DOS",
            "EMAIL_ROUTING",
            "EMAIL_SECURITY",
            "FW",
            "FW_PG",
            "HTTP",
            "HTTP_CONTROL",
            "HTTP_CRAWLER_REFERER",
            "HTTP_ORIGINS",
            "IQI",
            "LEAKED_CREDENTIALS",
            "NET",
            "ROBOTS_TXT",
            "SPEED",
            "WORKERS_AI",
        ]
        | Omit = omit,
        date_end: Union[str, datetime] | Omit = omit,
        date_range: str | Omit = omit,
        date_start: Union[str, datetime] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        geo_id: str | Omit = omit,
        limit: int | Omit = omit,
        location: str | Omit = omit,
        log: str | Omit = omit,
        offset: int | Omit = omit,
        origin: str | Omit = omit,
        outage_cause: Literal[
            "BLOCKING",
            "CABLE_CUT",
            "CYBERATTACK",
            "DNS",
            "FIRE",
            "GOVERNMENT_DIRECTED",
            "MAINTENANCE",
            "MECHANICAL",
            "MILITARY_ACTION",
            "MISCONFIGURATION",
            "NATURAL_DISASTER",
            "NETWORK_PROBLEM",
            "POWER_OUTAGE",
            "SOFTWARE",
            "TECHNICAL_PROBLEM",
            "UNKNOWN",
            "WEATHER",
        ]
        | Omit = omit,
        outage_type: Literal["NATIONWIDE", "REGIONAL", "NETWORK", "PLATFORM"] | Omit = omit,
        query: str | Omit = omit,
        tags: List[
            Literal[
                "ADM1",
                "ADM2",
                "API_TRAFFIC",
                "ARC",
                "AS",
                "ASN",
                "ATTACKS",
                "AUTHOR",
                "BANDWIDTH",
                "BITRATE",
                "BOT",
                "BOT_CATEGORY",
                "BOT_CLASS",
                "BOT_KIND",
                "BOT_OPERATOR",
                "BROWSER",
                "BROWSER_FAMILY",
                "BYTES",
                "CA",
                "CACHE_HIT",
                "CA_OWNER",
                "CHECK_RESULT",
                "CLIENT_TYPE",
                "COMPROMISED",
                "CONTENT_TYPE",
                "CRAWL_PURPOSE",
                "CRAWL_REFER_RATIO",
                "DEVICE_TYPE",
                "DKIM",
                "DMARC",
                "DNS",
                "DNSSEC",
                "DNSSEC_AWARE",
                "DNSSEC_E2E",
                "DOMAIN_CATEGORY",
                "DURATION",
                "EDNS",
                "ENCRYPTED",
                "ENTRY_TYPE",
                "EXPIRATION_STATUS",
                "HAS_IPS",
                "HAS_MATCHING_ANSWER",
                "HAS_WILDCARDS",
                "HTTP_METHOD",
                "HTTP_PROTOCOL",
                "HTTP_VERSION",
                "INDUSTRY",
                "IP_VERSION",
                "JITTER",
                "KEY_AGREEMENT",
                "LATENCY",
                "LOCATION",
                "LOCATION_LATENCY",
                "LOG",
                "LOG_API",
                "LOG_OPERATOR",
                "MALICIOUS",
                "MANAGED_RULES",
                "MITIGATION_PRODUCT",
                "MODEL",
                "NAMESERVER_LATENCY",
                "ORIGIN",
                "ORIGIN_AS",
                "ORIGIN_LOCATION",
                "ORIGIN_TARGET_LOCATION_PAIR",
                "OS",
                "PERCENTILE",
                "POST_QUANTUM",
                "PREFIX",
                "PRODUCT",
                "PROTOCOL",
                "PROVIDER",
                "PUBLIC_KEY_ALGORITHM",
                "QUERY_TYPE",
                "REFERER",
                "REGION",
                "RESPONSE_CODE",
                "RESPONSE_STATUS",
                "RESPONSE_STATUS_CATEGORY",
                "RESPONSE_TTL",
                "SIGNATURE_ALGORITHM",
                "SPAM",
                "SPF",
                "SPOOF",
                "SUCCESS_RATE",
                "TARGET_LOCATION",
                "TASK",
                "THREAT_CATEGORY",
                "TLD",
                "TLD_DNS_MAGNITUDE",
                "TLS_VERSION",
                "UPDATE_TYPE",
                "USER_AGENT",
                "VALIDATION_LEVEL",
                "VECTOR",
                "VERTICAL",
            ]
        ]
        | Omit = omit,
        tld: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OutageGetResponse:
        """
        Retrieves the latest Internet outages and anomalies.

        Args:
          asn: Filters results by Autonomous System. Specify a single Autonomous System Number
              (ASN) as integer.

          bot: Filters results by bot.

          ca: Filters results by certificate authority.

          data_source: Filters results by data source.

          date_end: End of the date range (inclusive). Alternative to `dateRange`; provide together
              with `dateStart`.

          date_range: Filters results by a relative date range ending at the current time. Use `<n>d`
              for days (up to `364d`) or `<n>w` for weeks (up to `52w`), e.g. `7d`. Append
              `control` to request the equivalent previous period for comparison: the
              comparison window is shifted back by the current window's length rounded up to a
              whole number of weeks, so it keeps the same weekday alignment and does not
              overlap the current window (e.g. `3dcontrol` covers days -10 to -7, `7dcontrol`
              covers days -14 to -7, `28dcontrol` covers days -56 to -28, and `10dcontrol`
              covers days -24 to -14). Mutually exclusive with `dateStart`/`dateEnd`.

          date_start: Start of the date range (inclusive). Alternative to `dateRange`; provide
              together with `dateEnd`.

          format: Format in which results will be returned.

          geo_id: Filters results by geolocation. Refer to
              [GeoNames](https://download.geonames.org/export/dump/readme.txt).

          limit: Limits the number of objects returned in the response.

          location: Filters results by location. Specify an alpha-2 location code.

          log: Filters results by certificate log.

          offset: Skips the specified number of objects before fetching the results.

          origin: Filters results by origin.

          outage_cause: Filters results by outage cause.

          outage_type: Filters results by outage type.

          query: Filters results by a free-text match on the annotation description, id, or
              linked entities (location, ASN, origin).

          tags: Filters results by annotation tag. Matches annotations carrying at least one of
              the given tags.

          tld: Filters results by top-level domain.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/annotations/outages",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "asn": asn,
                        "bot": bot,
                        "ca": ca,
                        "data_source": data_source,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "geo_id": geo_id,
                        "limit": limit,
                        "location": location,
                        "log": log,
                        "offset": offset,
                        "origin": origin,
                        "outage_cause": outage_cause,
                        "outage_type": outage_type,
                        "query": query,
                        "tags": tags,
                        "tld": tld,
                    },
                    outage_get_params.OutageGetParams,
                ),
                post_parser=ResultWrapper[OutageGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[OutageGetResponse], ResultWrapper[OutageGetResponse]),
        )

    async def locations(
        self,
        *,
        date_end: Union[str, datetime] | Omit = omit,
        date_range: str | Omit = omit,
        date_start: Union[str, datetime] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OutageLocationsResponse:
        """
        Retrieves the number of outages by location.

        Args:
          date_end: End of the date range (inclusive). Alternative to `dateRange`; provide together
              with `dateStart`.

          date_range: Filters results by a relative date range ending at the current time. Use `<n>d`
              for days (up to `364d`) or `<n>w` for weeks (up to `52w`), e.g. `7d`. Append
              `control` to request the equivalent previous period for comparison: the
              comparison window is shifted back by the current window's length rounded up to a
              whole number of weeks, so it keeps the same weekday alignment and does not
              overlap the current window (e.g. `3dcontrol` covers days -10 to -7, `7dcontrol`
              covers days -14 to -7, `28dcontrol` covers days -56 to -28, and `10dcontrol`
              covers days -24 to -14). Mutually exclusive with `dateStart`/`dateEnd`.

          date_start: Start of the date range (inclusive). Alternative to `dateRange`; provide
              together with `dateEnd`.

          format: Format in which results will be returned.

          limit: Limits the number of objects returned in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/annotations/outages/locations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit": limit,
                    },
                    outage_locations_params.OutageLocationsParams,
                ),
                post_parser=ResultWrapper[OutageLocationsResponse]._unwrapper,
            ),
            cast_to=cast(Type[OutageLocationsResponse], ResultWrapper[OutageLocationsResponse]),
        )


class OutagesResourceWithRawResponse:
    def __init__(self, outages: OutagesResource) -> None:
        self._outages = outages

        self.get = to_raw_response_wrapper(
            outages.get,
        )
        self.locations = to_raw_response_wrapper(
            outages.locations,
        )


class AsyncOutagesResourceWithRawResponse:
    def __init__(self, outages: AsyncOutagesResource) -> None:
        self._outages = outages

        self.get = async_to_raw_response_wrapper(
            outages.get,
        )
        self.locations = async_to_raw_response_wrapper(
            outages.locations,
        )


class OutagesResourceWithStreamingResponse:
    def __init__(self, outages: OutagesResource) -> None:
        self._outages = outages

        self.get = to_streamed_response_wrapper(
            outages.get,
        )
        self.locations = to_streamed_response_wrapper(
            outages.locations,
        )


class AsyncOutagesResourceWithStreamingResponse:
    def __init__(self, outages: AsyncOutagesResource) -> None:
        self._outages = outages

        self.get = async_to_streamed_response_wrapper(
            outages.get,
        )
        self.locations = async_to_streamed_response_wrapper(
            outages.locations,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["StoreCreateResponse"]


class StoreCreateResponse(BaseModel):
    id: str
    """Store Identifier."""

    created: datetime
    """When the secret was created."""

    modified: datetime
    """When the secret was modified."""

    name: str
    """The name of the store."""

    account_id: Optional[str] = None
    """Account Identifier."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = [
    "NamespaceUpdateParams",
    "PublicEndpointParams",
    "PublicEndpointParamsChatCompletionsEndpoint",
    "PublicEndpointParamsMcp",
    "PublicEndpointParamsRateLimit",
    "PublicEndpointParamsSearchEndpoint",
]


class NamespaceUpdateParams(TypedDict, total=False):
    account_id: Required[str]

    description: Optional[str]
    """Optional description for the namespace. Max 256 characters."""

    public_endpoint_params: Optional[PublicEndpointParams]


class PublicEndpointParamsChatCompletionsEndpoint(TypedDict, total=False):
    disabled: bool
    """Disable chat completions endpoint for this public endpoint"""


class PublicEndpointParamsMcp(TypedDict, total=False):
    description: str

    disabled: bool
    """Disable MCP endpoint for this public endpoint"""


class PublicEndpointParamsRateLimit(TypedDict, total=False):
    period_ms: int

    requests: int

    technique: Literal["fixed", "sliding"]


class PublicEndpointParamsSearchEndpoint(TypedDict, total=False):
    disabled: bool
    """Disable search endpoint for this public endpoint"""


class PublicEndpointParams(TypedDict, total=False):
    authorized_hosts: SequenceNotStr[str]

    chat_completions_endpoint: PublicEndpointParamsChatCompletionsEndpoint

    custom_domains: Optional[SequenceNotStr[str]]
    """Custom domain hostnames that alias this public endpoint.

    GET and create responses return the current set; on update (PUT) this field is
    only echoed back when supplied in the request body, otherwise it is null (omit
    it to leave domains unchanged).
    """

    default_domain_enabled: bool
    """
    When false, the instance is reachable only via a registered custom domain and
    the default <public_endpoint_id>.search.ai.cloudflare.com host returns 404.
    Requires at least one custom domain. Defaults to true. public_endpoint_params is
    replaced wholesale on update, so resend default_domain_enabled on every update
    to keep the default host off — omitting it resets to true.
    """

    enabled: bool

    instances_allowed: SequenceNotStr[str]
    """Instance IDs exposed through the namespace public endpoint.

    Empty means nothing is searchable. Every ID must be an existing instance in this
    namespace, and the list cannot exceed the account's multi-instance search limit.
    """

    mcp: PublicEndpointParamsMcp

    rate_limit: PublicEndpointParamsRateLimit

    search_endpoint: PublicEndpointParamsSearchEndpoint



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Annotated, TypedDict

from ....._utils import PropertyInfo

__all__ = ["ReportGetParams"]


class ReportGetParams(TypedDict, total=False):
    cdn_traffic: bool
    """Include CDN traffic in the bandwidth aggregation."""

    since: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Start of time interval to query, defaults to `until` - 6 hours.

    Timestamp must be in RFC3339 format and uses UTC unless otherwise specified.
    """

    until: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """End of time interval to query, defaults to current time.

    Timestamp must be in RFC3339 format and uses UTC unless otherwise specified.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ....._types import SequenceNotStr
from ....._utils import PropertyInfo

__all__ = ["TypeListParams"]


class TypeListParams(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    dataset_ids: Annotated[SequenceNotStr[str], PropertyInfo(alias="datasetIds")]
    """
    Dataset UUIDs to query, or one standalone scope value: 'all'/'\\**', 'analytics'
    for isAnalytics=true datasets, or 'operational' for isAnalytics=false datasets.
    If not provided, queries all accessible datasets.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["DatasetListResponse", "DatasetListResponseItem"]


class DatasetListResponseItem(BaseModel):
    indicator_write_mode: Literal["read_only", "create_only", "full"] = FieldInfo(alias="indicatorWriteMode")
    """
    Effective indicator mutation capability after account/dataset authorization and
    dataset storage capability are applied. API Gateway method permissions are
    separate and must also allow the requested operation.
    """

    is_analytics: bool = FieldInfo(alias="isAnalytics")

    is_public: bool = FieldInfo(alias="isPublic")

    name: str

    uuid: str

    deleted_at: Optional[str] = FieldInfo(alias="deletedAt", default=None)


DatasetListResponse: TypeAlias = List[DatasetListResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from datetime import datetime
from typing_extensions import Literal, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["AnnotationListParams"]


class AnnotationListParams(TypedDict, total=False):
    asn: int
    """Filters results by Autonomous System.

    Specify a single Autonomous System Number (ASN) as integer.
    """

    bot: str
    """Filters results by bot."""

    ca: str
    """Filters results by certificate authority."""

    data_source: Annotated[
        Literal[
            "ALL",
            "AI_BOTS",
            "AI_GATEWAY",
            "BGP",
            "BOTS",
            "CONNECTION_ANOMALY",
            "CT",
            "DNS",
            "DNS_MAGNITUDE",
            "DNS_AS112",
            "DOS",
            "EMAIL_ROUTING",
            "EMAIL_SECURITY",
            "FW",
            "FW_PG",
            "HTTP",
            "HTTP_CONTROL",
            "HTTP_CRAWLER_REFERER",
            "HTTP_ORIGINS",
            "IQI",
            "LEAKED_CREDENTIALS",
            "NET",
            "ROBOTS_TXT",
            "SPEED",
            "WORKERS_AI",
        ],
        PropertyInfo(alias="dataSource"),
    ]
    """Filters results by data source."""

    date_end: Annotated[Union[str, datetime], PropertyInfo(alias="dateEnd", format="iso8601")]
    """End of the date range (inclusive).

    Alternative to `dateRange`; provide together with `dateStart`.
    """

    date_range: Annotated[str, PropertyInfo(alias="dateRange")]
    """Filters results by a relative date range ending at the current time.

    Use `<n>d` for days (up to `364d`) or `<n>w` for weeks (up to `52w`), e.g. `7d`.
    Append `control` to request the equivalent previous period for comparison: the
    comparison window is shifted back by the current window's length rounded up to a
    whole number of weeks, so it keeps the same weekday alignment and does not
    overlap the current window (e.g. `3dcontrol` covers days -10 to -7, `7dcontrol`
    covers days -14 to -7, `28dcontrol` covers days -56 to -28, and `10dcontrol`
    covers days -24 to -14). Mutually exclusive with `dateStart`/`dateEnd`.
    """

    date_start: Annotated[Union[str, datetime], PropertyInfo(alias="dateStart", format="iso8601")]
    """Start of the date range (inclusive).

    Alternative to `dateRange`; provide together with `dateEnd`.
    """

    event_type: Annotated[
        Literal["EVENT", "GENERAL", "OUTAGE", "PARTIAL_PROJECTION", "PIPELINE", "TRAFFIC_ANOMALY"],
        PropertyInfo(alias="eventType"),
    ]
    """Filters results by event type. EVENT is a legacy alias for GENERAL."""

    format: Literal["JSON", "CSV"]
    """Format in which results will be returned."""

    geo_id: Annotated[str, PropertyInfo(alias="geoId")]
    """Filters results by geolocation.

    Refer to [GeoNames](https://download.geonames.org/export/dump/readme.txt).
    """

    limit: int
    """Limits the number of objects returned in the response."""

    location: str
    """Filters results by location. Specify an alpha-2 location code."""

    log: str
    """Filters results by certificate log."""

    offset: int
    """Skips the specified number of objects before fetching the results."""

    origin: str
    """Filters results by origin."""

    outage_cause: Annotated[
        Literal[
            "BLOCKING",
            "CABLE_CUT",
            "CYBERATTACK",
            "DNS",
            "FIRE",
            "GOVERNMENT_DIRECTED",
            "MAINTENANCE",
            "MECHANICAL",
            "MILITARY_ACTION",
            "MISCONFIGURATION",
            "NATURAL_DISASTER",
            "NETWORK_PROBLEM",
            "POWER_OUTAGE",
            "SOFTWARE",
            "TECHNICAL_PROBLEM",
            "UNKNOWN",
            "WEATHER",
        ],
        PropertyInfo(alias="outageCause"),
    ]
    """Filters results by outage cause."""

    outage_type: Annotated[Literal["NATIONWIDE", "REGIONAL", "NETWORK", "PLATFORM"], PropertyInfo(alias="outageType")]
    """Filters results by outage type."""

    query: str
    """
    Filters results by a free-text match on the annotation description, id, or
    linked entities (location, ASN, origin).
    """

    tags: List[
        Literal[
            "ADM1",
            "ADM2",
            "API_TRAFFIC",
            "ARC",
            "AS",
            "ASN",
            "ATTACKS",
            "AUTHOR",
            "BANDWIDTH",
            "BITRATE",
            "BOT",
            "BOT_CATEGORY",
            "BOT_CLASS",
            "BOT_KIND",
            "BOT_OPERATOR",
            "BROWSER",
            "BROWSER_FAMILY",
            "BYTES",
            "CA",
            "CACHE_HIT",
            "CA_OWNER",
            "CHECK_RESULT",
            "CLIENT_TYPE",
            "COMPROMISED",
            "CONTENT_TYPE",
            "CRAWL_PURPOSE",
            "CRAWL_REFER_RATIO",
            "DEVICE_TYPE",
            "DKIM",
            "DMARC",
            "DNS",
            "DNSSEC",
            "DNSSEC_AWARE",
            "DNSSEC_E2E",
            "DOMAIN_CATEGORY",
            "DURATION",
            "EDNS",
            "ENCRYPTED",
            "ENTRY_TYPE",
            "EXPIRATION_STATUS",
            "HAS_IPS",
            "HAS_MATCHING_ANSWER",
            "HAS_WILDCARDS",
            "HTTP_METHOD",
            "HTTP_PROTOCOL",
            "HTTP_VERSION",
            "INDUSTRY",
            "IP_VERSION",
            "JITTER",
            "KEY_AGREEMENT",
            "LATENCY",
            "LOCATION",
            "LOCATION_LATENCY",
            "LOG",
            "LOG_API",
            "LOG_OPERATOR",
            "MALICIOUS",
            "MANAGED_RULES",
            "MITIGATION_PRODUCT",
            "MODEL",
            "NAMESERVER_LATENCY",
            "ORIGIN",
            "ORIGIN_AS",
            "ORIGIN_LOCATION",
            "ORIGIN_TARGET_LOCATION_PAIR",
            "OS",
            "PERCENTILE",
            "POST_QUANTUM",
            "PREFIX",
            "PRODUCT",
            "PROTOCOL",
            "PROVIDER",
            "PUBLIC_KEY_ALGORITHM",
            "QUERY_TYPE",
            "REFERER",
            "REGION",
            "RESPONSE_CODE",
            "RESPONSE_STATUS",
            "RESPONSE_STATUS_CATEGORY",
            "RESPONSE_TTL",
            "SIGNATURE_ALGORITHM",
            "SPAM",
            "SPF",
            "SPOOF",
            "SUCCESS_RATE",
            "TARGET_LOCATION",
            "TASK",
            "THREAT_CATEGORY",
            "TLD",
            "TLD_DNS_MAGNITUDE",
            "TLS_VERSION",
            "UPDATE_TYPE",
            "USER_AGENT",
            "VALIDATION_LEVEL",
            "VECTOR",
            "VERTICAL",
        ]
    ]
    """Filters results by annotation tag.

    Matches annotations carrying at least one of the given tags.
    """

    tld: str
    """Filters results by top-level domain."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["InstanceStepResponse", "Error"]


class Error(BaseModel):
    """Error details when status='errored'; null otherwise."""

    message: str

    name: str


class InstanceStepResponse(BaseModel):
    error: Optional[Error] = None
    """Error details when status='errored'; null otherwise."""

    status: Literal[
        "queued", "running", "paused", "errored", "terminated", "complete", "waitingForPause", "waiting", "rollingBack"
    ]

    event_type: Optional[str] = None
    """The event type the step is waiting on, as supplied to step.waitForEvent.

    Only present when type='waitForEvent'.
    """

    output: Optional[object] = None
    """Contains the full step output or waitForEvent payload without truncation.

    Uses '[REDACTED]' for sensitive outputs. Contains a value when
    status='complete'. May contain a ReadableStream when step.do returns one; the
    response serves stream outputs as application/octet-stream rather than JSON.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .content import (
    ContentResource,
    AsyncContentResource,
    ContentResourceWithRawResponse,
    AsyncContentResourceWithRawResponse,
    ContentResourceWithStreamingResponse,
    AsyncContentResourceWithStreamingResponse,
)
from .versions import (
    VersionsResource,
    AsyncVersionsResource,
    VersionsResourceWithRawResponse,
    AsyncVersionsResourceWithRawResponse,
    VersionsResourceWithStreamingResponse,
    AsyncVersionsResourceWithStreamingResponse,
)
from .transformers import (
    TransformersResource,
    AsyncTransformersResource,
    TransformersResourceWithRawResponse,
    AsyncTransformersResourceWithRawResponse,
    TransformersResourceWithStreamingResponse,
    AsyncTransformersResourceWithStreamingResponse,
)

__all__ = [
    "ContentResource",
    "AsyncContentResource",
    "ContentResourceWithRawResponse",
    "AsyncContentResourceWithRawResponse",
    "ContentResourceWithStreamingResponse",
    "AsyncContentResourceWithStreamingResponse",
    "VersionsResource",
    "AsyncVersionsResource",
    "VersionsResourceWithRawResponse",
    "AsyncVersionsResourceWithRawResponse",
    "VersionsResourceWithStreamingResponse",
    "AsyncVersionsResourceWithStreamingResponse",
    "TransformersResource",
    "AsyncTransformersResource",
    "TransformersResourceWithRawResponse",
    "AsyncTransformersResourceWithRawResponse",
    "TransformersResourceWithStreamingResponse",
    "AsyncTransformersResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Type, Union, Optional, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.moq.relays import token_create_params
from ....types.moq.relays.token_list_response import TokenListResponse
from ....types.moq.relays.token_create_response import TokenCreateResponse
from ....types.moq.relays.token_delete_response import TokenDeleteResponse

__all__ = ["TokensResource", "AsyncTokensResource"]


class TokensResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TokensResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TokensResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TokensResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TokensResourceWithStreamingResponse(self)

    def create(
        self,
        relay_id: str,
        *,
        account_id: str,
        operations: List[Literal["publish", "subscribe"]],
        expires: Union[str, datetime] | Omit = omit,
        label: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TokenCreateResponse]:
        """Mints a new relay-scoped token and adds it to the relay's accepted-auth
        registry.

        The token value (secret) is shown once in the response. A relay may
        hold up to 10 tokens; creating an 11th is rejected.

        Args:
          account_id: Cloudflare account identifier.

          operations: Non-empty subset of the V1 roles the token is allowed to perform. Signed into
              the token.

          expires: Optional expiry (RFC 3339). Defaults to 1 year from creation; rejected if more
              than 1 year in the future.

          label: Optional, customer-set label.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not relay_id:
            raise ValueError(f"Expected a non-empty value for `relay_id` but received {relay_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/moq/relays/{relay_id}/tokens", account_id=account_id, relay_id=relay_id
            ),
            body=maybe_transform(
                {
                    "operations": operations,
                    "expires": expires,
                    "label": label,
                },
                token_create_params.TokenCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TokenCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TokenCreateResponse]], ResultWrapper[TokenCreateResponse]),
        )

    def list(
        self,
        relay_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TokenListResponse]:
        """Returns metadata for every token the relay accepts.

        Secrets are never returned,
        so a token that has been lost cannot be recovered here. There is no expiry
        filter: compare each token's `expires` to the current time to tell which ones
        have lapsed.

        Args:
          account_id: Cloudflare account identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not relay_id:
            raise ValueError(f"Expected a non-empty value for `relay_id` but received {relay_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/moq/relays/{relay_id}/tokens", account_id=account_id, relay_id=relay_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TokenListResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TokenListResponse]], ResultWrapper[TokenListResponse]),
        )

    def delete(
        self,
        jti: str,
        *,
        account_id: str,
        relay_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TokenDeleteResponse:
        """Revokes a token by removing it from the set the relay accepts.

        Relays cache that
        set, so revocation takes effect within seconds rather than instantly, and
        connections already established with the token are not closed. Revoking an
        unknown token succeeds, so the call is idempotent.

        Args:
          account_id: Cloudflare account identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not relay_id:
            raise ValueError(f"Expected a non-empty value for `relay_id` but received {relay_id!r}")
        if not jti:
            raise ValueError(f"Expected a non-empty value for `jti` but received {jti!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/moq/relays/{relay_id}/tokens/{jti}",
                account_id=account_id,
                relay_id=relay_id,
                jti=jti,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TokenDeleteResponse,
        )


class AsyncTokensResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTokensResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTokensResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTokensResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTokensResourceWithStreamingResponse(self)

    async def create(
        self,
        relay_id: str,
        *,
        account_id: str,
        operations: List[Literal["publish", "subscribe"]],
        expires: Union[str, datetime] | Omit = omit,
        label: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TokenCreateResponse]:
        """Mints a new relay-scoped token and adds it to the relay's accepted-auth
        registry.

        The token value (secret) is shown once in the response. A relay may
        hold up to 10 tokens; creating an 11th is rejected.

        Args:
          account_id: Cloudflare account identifier.

          operations: Non-empty subset of the V1 roles the token is allowed to perform. Signed into
              the token.

          expires: Optional expiry (RFC 3339). Defaults to 1 year from creation; rejected if more
              than 1 year in the future.

          label: Optional, customer-set label.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not relay_id:
            raise ValueError(f"Expected a non-empty value for `relay_id` but received {relay_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/moq/relays/{relay_id}/tokens", account_id=account_id, relay_id=relay_id
            ),
            body=await async_maybe_transform(
                {
                    "operations": operations,
                    "expires": expires,
                    "label": label,
                },
                token_create_params.TokenCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TokenCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TokenCreateResponse]], ResultWrapper[TokenCreateResponse]),
        )

    async def list(
        self,
        relay_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TokenListResponse]:
        """Returns metadata for every token the relay accepts.

        Secrets are never returned,
        so a token that has been lost cannot be recovered here. There is no expiry
        filter: compare each token's `expires` to the current time to tell which ones
        have lapsed.

        Args:
          account_id: Cloudflare account identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not relay_id:
            raise ValueError(f"Expected a non-empty value for `relay_id` but received {relay_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/moq/relays/{relay_id}/tokens", account_id=account_id, relay_id=relay_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TokenListResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TokenListResponse]], ResultWrapper[TokenListResponse]),
        )

    async def delete(
        self,
        jti: str,
        *,
        account_id: str,
        relay_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TokenDeleteResponse:
        """Revokes a token by removing it from the set the relay accepts.

        Relays cache that
        set, so revocation takes effect within seconds rather than instantly, and
        connections already established with the token are not closed. Revoking an
        unknown token succeeds, so the call is idempotent.

        Args:
          account_id: Cloudflare account identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not relay_id:
            raise ValueError(f"Expected a non-empty value for `relay_id` but received {relay_id!r}")
        if not jti:
            raise ValueError(f"Expected a non-empty value for `jti` but received {jti!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/moq/relays/{relay_id}/tokens/{jti}",
                account_id=account_id,
                relay_id=relay_id,
                jti=jti,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TokenDeleteResponse,
        )


class TokensResourceWithRawResponse:
    def __init__(self, tokens: TokensResource) -> None:
        self._tokens = tokens

        self.create = to_raw_response_wrapper(
            tokens.create,
        )
        self.list = to_raw_response_wrapper(
            tokens.list,
        )
        self.delete = to_raw_response_wrapper(
            tokens.delete,
        )


class AsyncTokensResourceWithRawResponse:
    def __init__(self, tokens: AsyncTokensResource) -> None:
        self._tokens = tokens

        self.create = async_to_raw_response_wrapper(
            tokens.create,
        )
        self.list = async_to_raw_response_wrapper(
            tokens.list,
        )
        self.delete = async_to_raw_response_wrapper(
            tokens.delete,
        )


class TokensResourceWithStreamingResponse:
    def __init__(self, tokens: TokensResource) -> None:
        self._tokens = tokens

        self.create = to_streamed_response_wrapper(
            tokens.create,
        )
        self.list = to_streamed_response_wrapper(
            tokens.list,
        )
        self.delete = to_streamed_response_wrapper(
            tokens.delete,
        )


class AsyncTokensResourceWithStreamingResponse:
    def __init__(self, tokens: AsyncTokensResource) -> None:
        self._tokens = tokens

        self.create = async_to_streamed_response_wrapper(
            tokens.create,
        )
        self.list = async_to_streamed_response_wrapper(
            tokens.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            tokens.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Required, TypedDict

__all__ = ["TailCreateParams"]


class TailCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    project_name: Required[str]
    """Name of the project."""

    filters: Iterable[Dict[str, object]]
    """Filters to apply to the tail session."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .tail_create_params import TailCreateParams as TailCreateParams
from .tail_create_response import TailCreateResponse as TailCreateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .zsk import (
    ZskResource,
    AsyncZskResource,
    ZskResourceWithRawResponse,
    AsyncZskResourceWithRawResponse,
    ZskResourceWithStreamingResponse,
    AsyncZskResourceWithStreamingResponse,
)
from .dnssec import (
    DNSSECResource,
    AsyncDNSSECResource,
    DNSSECResourceWithRawResponse,
    AsyncDNSSECResourceWithRawResponse,
    DNSSECResourceWithStreamingResponse,
    AsyncDNSSECResourceWithStreamingResponse,
)

__all__ = [
    "ZskResource",
    "AsyncZskResource",
    "ZskResourceWithRawResponse",
    "AsyncZskResourceWithRawResponse",
    "ZskResourceWithStreamingResponse",
    "AsyncZskResourceWithStreamingResponse",
    "DNSSECResource",
    "AsyncDNSSECResource",
    "DNSSECResourceWithRawResponse",
    "AsyncDNSSECResourceWithRawResponse",
    "DNSSECResourceWithStreamingResponse",
    "AsyncDNSSECResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ...._models import BaseModel

__all__ = ["TransformationsConfig"]


class TransformationsConfig(BaseModel):
    """A configuration item for a specific zone and feature."""

    id: Optional[str] = None
    """Feature identifier."""

    cf_zone_tag: Optional[str] = None
    """Zone tag identifier."""

    editable: Optional[bool] = None
    """Whether this setting can be modified."""

    modified_on: Optional[datetime] = None
    """When this setting was last modified."""

    value: Optional[str] = None
    """Current value of the feature setting."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["TransformerUpdateResponse", "AssociatedJob"]


class AssociatedJob(BaseModel):
    id: Optional[int] = None
    """The logpush job ID."""

    name: Optional[str] = None
    """The logpush job destination name."""

    object_tag: Optional[str] = None
    """The zone or account tag."""

    object_type: Optional[Literal["zone", "account"]] = None
    """Whether the job is zone-scoped or account-scoped."""


class TransformerUpdateResponse(BaseModel):
    id: Optional[int] = None
    """The transformer ID."""

    associated_jobs: Optional[List[AssociatedJob]] = None
    """Logpush jobs that reference this transformer."""

    created_at: Optional[datetime] = None
    """When the transformer was created (RFC 3339)."""

    dataset: Optional[str] = None
    """
    The dataset this transformer operates on, derived from the SQL query's FROM
    clause. Informational only. May be absent if the dataset cannot be determined
    from the query.
    """

    description: Optional[str] = None
    """Optional customer-provided description."""

    name: Optional[str] = None
    """Customer-provided name for identification."""

    updated_at: Optional[datetime] = None
    """When the transformer was last modified (RFC 3339)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Union, Iterable, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncCursorPagination, AsyncCursorPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.email_sending import (
    suppression_edit_params,
    suppression_list_params,
    suppression_create_params,
    suppression_import_params,
)
from ...types.email_sending.suppression_get_response import SuppressionGetResponse
from ...types.email_sending.suppression_edit_response import SuppressionEditResponse
from ...types.email_sending.suppression_list_response import SuppressionListResponse
from ...types.email_sending.suppression_create_response import SuppressionCreateResponse
from ...types.email_sending.suppression_delete_response import SuppressionDeleteResponse
from ...types.email_sending.suppression_import_response import SuppressionImportResponse

__all__ = ["SuppressionsResource", "AsyncSuppressionsResource"]


class SuppressionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SuppressionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SuppressionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SuppressionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SuppressionsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        email: str,
        expires_at: Union[str, datetime, None] | Omit = omit,
        note: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SuppressionCreateResponse:
        """Creates an account-wide suppression.

        If a mutable legacy zone-linked row already
        exists, it is promoted without changing its identifier.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/email/sending/suppressions", account_id=account_id),
            body=maybe_transform(
                {
                    "email": email,
                    "expires_at": expires_at,
                    "note": note,
                },
                suppression_create_params.SuppressionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SuppressionCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[SuppressionCreateResponse], ResultWrapper[SuppressionCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        cursor: str | Omit = omit,
        email: str | Omit = omit,
        per_page: int | Omit = omit,
        reason: Literal["manual", "complaint", "hard_bounce", "soft_bounce", "policy"] | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPagination[SuppressionListResponse]:
        """
        Lists every active Email Sending suppression owned by the account, including
        legacy rows with internal zone memberships.

        Args:
          cursor: Opaque pagination cursor returned as `result_info.next_cursor`. It carries the
              filters that produced it.

          email: Exact email-address filter.

          search: A complete address is an exact match; a value ending in `@` matches that
              username across every domain. Prefix searches may return short intermediate
              pages while the bounded account scan advances.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/email/sending/suppressions", account_id=account_id),
            page=SyncCursorPagination[SuppressionListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "email": email,
                        "per_page": per_page,
                        "reason": reason,
                        "search": search,
                    },
                    suppression_list_params.SuppressionListParams,
                ),
            ),
            model=SuppressionListResponse,
        )

    def delete(
        self,
        suppression_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SuppressionDeleteResponse:
        """
        Deletes the suppression, its note, and every legacy internal zone membership,
        allowing future delivery attempts to the address.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not suppression_id:
            raise ValueError(f"Expected a non-empty value for `suppression_id` but received {suppression_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/email/sending/suppressions/{suppression_id}",
                account_id=account_id,
                suppression_id=suppression_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SuppressionDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[SuppressionDeleteResponse], ResultWrapper[SuppressionDeleteResponse]),
        )

    def edit(
        self,
        suppression_id: str,
        *,
        account_id: str,
        expires_at: Union[str, datetime, None] | Omit = omit,
        note: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SuppressionEditResponse:
        """
        Updates expiry or advisory note fields without changing legacy internal zone
        memberships.

        Args:
          expires_at: New expiry. Send `null` to make the suppression permanent; omit to leave it
              unchanged.

          note: Replacement advisory note. Send an empty string to clear it; omit to leave it
              unchanged.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not suppression_id:
            raise ValueError(f"Expected a non-empty value for `suppression_id` but received {suppression_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/email/sending/suppressions/{suppression_id}",
                account_id=account_id,
                suppression_id=suppression_id,
            ),
            body=maybe_transform(
                {
                    "expires_at": expires_at,
                    "note": note,
                },
                suppression_edit_params.SuppressionEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SuppressionEditResponse]._unwrapper,
            ),
            cast_to=cast(Type[SuppressionEditResponse], ResultWrapper[SuppressionEditResponse]),
        )

    def get(
        self,
        suppression_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SuppressionGetResponse:
        """
        Gets an Email Sending suppression owned by the account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not suppression_id:
            raise ValueError(f"Expected a non-empty value for `suppression_id` but received {suppression_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/email/sending/suppressions/{suppression_id}",
                account_id=account_id,
                suppression_id=suppression_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SuppressionGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[SuppressionGetResponse], ResultWrapper[SuppressionGetResponse]),
        )

    def import_(
        self,
        *,
        account_id: str,
        items: Iterable[suppression_import_params.Item],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SuppressionImportResponse:
        """
        Imports up to 1,000 account-level Email Sending suppressions in one request.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/email/sending/suppressions/bulk", account_id=account_id),
            body=maybe_transform({"items": items}, suppression_import_params.SuppressionImportParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SuppressionImportResponse]._unwrapper,
            ),
            cast_to=cast(Type[SuppressionImportResponse], ResultWrapper[SuppressionImportResponse]),
        )


class AsyncSuppressionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSuppressionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSuppressionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSuppressionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSuppressionsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        email: str,
        expires_at: Union[str, datetime, None] | Omit = omit,
        note: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SuppressionCreateResponse:
        """Creates an account-wide suppression.

        If a mutable legacy zone-linked row already
        exists, it is promoted without changing its identifier.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/email/sending/suppressions", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "email": email,
                    "expires_at": expires_at,
                    "note": note,
                },
                suppression_create_params.SuppressionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SuppressionCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[SuppressionCreateResponse], ResultWrapper[SuppressionCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        cursor: str | Omit = omit,
        email: str | Omit = omit,
        per_page: int | Omit = omit,
        reason: Literal["manual", "complaint", "hard_bounce", "soft_bounce", "policy"] | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[SuppressionListResponse, AsyncCursorPagination[SuppressionListResponse]]:
        """
        Lists every active Email Sending suppression owned by the account, including
        legacy rows with internal zone memberships.

        Args:
          cursor: Opaque pagination cursor returned as `result_info.next_cursor`. It carries the
              filters that produced it.

          email: Exact email-address filter.

          search: A complete address is an exact match; a value ending in `@` matches that
              username across every domain. Prefix searches may return short intermediate
              pages while the bounded account scan advances.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/email/sending/suppressions", account_id=account_id),
            page=AsyncCursorPagination[SuppressionListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "email": email,
                        "per_page": per_page,
                        "reason": reason,
                        "search": search,
                    },
                    suppression_list_params.SuppressionListParams,
                ),
            ),
            model=SuppressionListResponse,
        )

    async def delete(
        self,
        suppression_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SuppressionDeleteResponse:
        """
        Deletes the suppression, its note, and every legacy internal zone membership,
        allowing future delivery attempts to the address.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not suppression_id:
            raise ValueError(f"Expected a non-empty value for `suppression_id` but received {suppression_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/email/sending/suppressions/{suppression_id}",
                account_id=account_id,
                suppression_id=suppression_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SuppressionDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[SuppressionDeleteResponse], ResultWrapper[SuppressionDeleteResponse]),
        )

    async def edit(
        self,
        suppression_id: str,
        *,
        account_id: str,
        expires_at: Union[str, datetime, None] | Omit = omit,
        note: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SuppressionEditResponse:
        """
        Updates expiry or advisory note fields without changing legacy internal zone
        memberships.

        Args:
          expires_at: New expiry. Send `null` to make the suppression permanent; omit to leave it
              unchanged.

          note: Replacement advisory note. Send an empty string to clear it; omit to leave it
              unchanged.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not suppression_id:
            raise ValueError(f"Expected a non-empty value for `suppression_id` but received {suppression_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/email/sending/suppressions/{suppression_id}",
                account_id=account_id,
                suppression_id=suppression_id,
            ),
            body=await async_maybe_transform(
                {
                    "expires_at": expires_at,
                    "note": note,
                },
                suppression_edit_params.SuppressionEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SuppressionEditResponse]._unwrapper,
            ),
            cast_to=cast(Type[SuppressionEditResponse], ResultWrapper[SuppressionEditResponse]),
        )

    async def get(
        self,
        suppression_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SuppressionGetResponse:
        """
        Gets an Email Sending suppression owned by the account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not suppression_id:
            raise ValueError(f"Expected a non-empty value for `suppression_id` but received {suppression_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/email/sending/suppressions/{suppression_id}",
                account_id=account_id,
                suppression_id=suppression_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SuppressionGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[SuppressionGetResponse], ResultWrapper[SuppressionGetResponse]),
        )

    async def import_(
        self,
        *,
        account_id: str,
        items: Iterable[suppression_import_params.Item],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SuppressionImportResponse:
        """
        Imports up to 1,000 account-level Email Sending suppressions in one request.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/email/sending/suppressions/bulk", account_id=account_id),
            body=await async_maybe_transform({"items": items}, suppression_import_params.SuppressionImportParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SuppressionImportResponse]._unwrapper,
            ),
            cast_to=cast(Type[SuppressionImportResponse], ResultWrapper[SuppressionImportResponse]),
        )


class SuppressionsResourceWithRawResponse:
    def __init__(self, suppressions: SuppressionsResource) -> None:
        self._suppressions = suppressions

        self.create = to_raw_response_wrapper(
            suppressions.create,
        )
        self.list = to_raw_response_wrapper(
            suppressions.list,
        )
        self.delete = to_raw_response_wrapper(
            suppressions.delete,
        )
        self.edit = to_raw_response_wrapper(
            suppressions.edit,
        )
        self.get = to_raw_response_wrapper(
            suppressions.get,
        )
        self.import_ = to_raw_response_wrapper(
            suppressions.import_,
        )


class AsyncSuppressionsResourceWithRawResponse:
    def __init__(self, suppressions: AsyncSuppressionsResource) -> None:
        self._suppressions = suppressions

        self.create = async_to_raw_response_wrapper(
            suppressions.create,
        )
        self.list = async_to_raw_response_wrapper(
            suppressions.list,
        )
        self.delete = async_to_raw_response_wrapper(
            suppressions.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            suppressions.edit,
        )
        self.get = async_to_raw_response_wrapper(
            suppressions.get,
        )
        self.import_ = async_to_raw_response_wrapper(
            suppressions.import_,
        )


class SuppressionsResourceWithStreamingResponse:
    def __init__(self, suppressions: SuppressionsResource) -> None:
        self._suppressions = suppressions

        self.create = to_streamed_response_wrapper(
            suppressions.create,
        )
        self.list = to_streamed_response_wrapper(
            suppressions.list,
        )
        self.delete = to_streamed_response_wrapper(
            suppressions.delete,
        )
        self.edit = to_streamed_response_wrapper(
            suppressions.edit,
        )
        self.get = to_streamed_response_wrapper(
            suppressions.get,
        )
        self.import_ = to_streamed_response_wrapper(
            suppressions.import_,
        )


class AsyncSuppressionsResourceWithStreamingResponse:
    def __init__(self, suppressions: AsyncSuppressionsResource) -> None:
        self._suppressions = suppressions

        self.create = async_to_streamed_response_wrapper(
            suppressions.create,
        )
        self.list = async_to_streamed_response_wrapper(
            suppressions.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            suppressions.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            suppressions.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            suppressions.get,
        )
        self.import_ = async_to_streamed_response_wrapper(
            suppressions.import_,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import date
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["UsageGetAccountUsageV1Params"]


class UsageGetAccountUsageV1Params(TypedDict, total=False):
    account_id: Required[str]
    """Represents a Cloudflare resource identifier tag."""

    from_: Annotated[Union[str, date], PropertyInfo(alias="from", format="iso8601")]
    """Start date for the usage query (ISO 8601).

    The provided time range must include the subscription billing cycle anchor day,
    otherwise no usage data is returned. Use the info endpoint to retrieve the
    subscription anchor day.
    """

    to: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """End date for the usage query (ISO 8601)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = [
    "Worker",
    "Observability",
    "ObservabilityLogs",
    "ObservabilityTraces",
    "References",
    "ReferencesDispatchNamespaceOutbound",
    "ReferencesDomain",
    "ReferencesDurableObject",
    "ReferencesQueue",
    "ReferencesWorker",
    "Subdomain",
    "TailConsumer",
]


class ObservabilityLogs(BaseModel):
    """Log settings for the Worker."""

    destinations: Optional[List[str]] = None
    """A list of destinations where logs will be exported to."""

    enabled: Optional[bool] = None
    """Whether logs are enabled for the Worker."""

    head_sampling_rate: Optional[float] = None
    """The sampling rate for logs. From 0 to 1 (1 = 100%, 0.1 = 10%)."""

    invocation_logs: Optional[bool] = None
    """
    Whether
    [invocation logs](https://developers.cloudflare.com/workers/observability/logs/workers-logs/#invocation-logs)
    are enabled for the Worker.
    """

    persist: Optional[bool] = None
    """Whether log persistence is enabled for the Worker."""


class ObservabilityTraces(BaseModel):
    """Trace settings for the Worker."""

    destinations: Optional[List[str]] = None
    """A list of destinations where traces will be exported to."""

    enabled: Optional[bool] = None
    """Whether traces are enabled for the Worker."""

    head_sampling_rate: Optional[float] = None
    """The sampling rate for traces. From 0 to 1 (1 = 100%, 0.1 = 10%)."""

    persist: Optional[bool] = None
    """Whether trace persistence is enabled for the Worker."""

    propagation_policy: Optional[Literal["authenticated", "accept"]] = None
    """
    Controls how inbound trace context (traceparent/tracestate) headers on incoming
    requests are handled. "authenticated" honors inbound trace context only when
    accompanied by a valid trace auth token. "accept" unconditionally accepts
    inbound trace context. Requires the trace propagation feature to be enabled.
    Returns null when the trace propagation feature is not enabled for the account.
    """


class Observability(BaseModel):
    """Observability settings for the Worker."""

    enabled: Optional[bool] = None
    """Whether observability is enabled for the Worker."""

    head_sampling_rate: Optional[float] = None
    """The sampling rate for observability. From 0 to 1 (1 = 100%, 0.1 = 10%)."""

    logs: Optional[ObservabilityLogs] = None
    """Log settings for the Worker."""

    redact_query_string: Optional[bool] = None
    """Whether query strings are removed from request URLs in logs and traces."""

    traces: Optional[ObservabilityTraces] = None
    """Trace settings for the Worker."""


class ReferencesDispatchNamespaceOutbound(BaseModel):
    namespace_id: str
    """ID of the dispatch namespace."""

    namespace_name: str
    """Name of the dispatch namespace."""

    worker_id: str
    """ID of the Worker using the dispatch namespace."""

    worker_name: str
    """Name of the Worker using the dispatch namespace."""


class ReferencesDomain(BaseModel):
    id: str
    """ID of the custom domain."""

    certificate_id: str
    """ID of the TLS certificate issued for the custom domain."""

    hostname: str
    """Full hostname of the custom domain, including the zone name."""

    zone_id: str
    """ID of the zone."""

    zone_name: str
    """Name of the zone."""


class ReferencesDurableObject(BaseModel):
    namespace_id: str
    """ID of the Durable Object namespace being used."""

    namespace_name: str
    """Name of the Durable Object namespace being used."""

    worker_id: str
    """ID of the Worker using the Durable Object implementation."""

    worker_name: str
    """Name of the Worker using the Durable Object implementation."""


class ReferencesQueue(BaseModel):
    queue_consumer_id: str
    """ID of the queue consumer configuration."""

    queue_id: str
    """ID of the queue."""

    queue_name: str
    """Name of the queue."""


class ReferencesWorker(BaseModel):
    id: str
    """ID of the referencing Worker."""

    name: str
    """Name of the referencing Worker."""


class References(BaseModel):
    """Other resources that reference the Worker and depend on it existing."""

    dispatch_namespace_outbounds: List[ReferencesDispatchNamespaceOutbound]
    """
    Other Workers that reference the Worker as an outbound for a dispatch namespace.
    """

    domains: List[ReferencesDomain]
    """Custom domains connected to the Worker."""

    durable_objects: List[ReferencesDurableObject]
    """Other Workers that reference Durable Object classes implemented by the Worker."""

    queues: List[ReferencesQueue]
    """Queues that send messages to the Worker."""

    workers: List[ReferencesWorker]
    """
    Other Workers that reference the Worker using
    [service bindings](https://developers.cloudflare.com/workers/runtime-apis/bindings/service-bindings/).
    """


class Subdomain(BaseModel):
    """Subdomain settings for the Worker."""

    enabled: Optional[bool] = None
    """Whether the \\**.workers.dev subdomain is enabled for the Worker."""

    preview_url_suffix: Optional[str] = None
    """
    Prepend a version or preview prefix to this host suffix to form the
    \\**.workers.dev
    [preview URL](https://developers.cloudflare.com/workers/configuration/previews/)
    the Worker would serve on once previews are enabled, e.g.
    `https://<prefix>-my-worker.my-subdomain.workers.dev`. Present whenever the
    account owns a workers.dev subdomain, regardless of whether `previews_enabled`
    is true, so presence does not imply preview URLs are currently live. Absent only
    when the account owns no workers.dev subdomain.
    """

    previews_enabled: Optional[bool] = None
    """
    Whether
    [preview URLs](https://developers.cloudflare.com/workers/configuration/previews/)
    are enabled for the Worker.
    """

    url: Optional[str] = None
    """
    The address the Worker would serve on once its \\**.workers.dev subdomain is
    enabled. Present whenever the account owns a workers.dev subdomain, regardless
    of whether `enabled` is true, so presence does not imply the Worker is currently
    live at this URL. Absent only when the account owns no workers.dev subdomain.
    """


class TailConsumer(BaseModel):
    name: str
    """Name of the consumer Worker."""


class Worker(BaseModel):
    id: str
    """Immutable ID of the Worker."""

    created_on: datetime
    """When the Worker was created."""

    logpush: bool
    """Whether logpush is enabled for the Worker."""

    name: str
    """Name of the Worker."""

    observability: Observability
    """Observability settings for the Worker."""

    references: References
    """Other resources that reference the Worker and depend on it existing."""

    subdomain: Subdomain
    """Subdomain settings for the Worker."""

    tags: List[str]
    """Tags associated with the Worker."""

    tail_consumers: List[TailConsumer]
    """Other Workers that should consume logs from the Worker."""

    updated_on: datetime
    """When the Worker was most recently updated."""

    deployed_on: Optional[datetime] = None
    """When the Worker's most recent deployment was created.

    `null` if the Worker has never been deployed.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["Registration"]


class Registration(BaseModel):
    """
    A domain registration resource representing the current state of a registered domain.
    """

    auto_renew: bool
    """Whether automatic renewal occurs before expiration."""

    created_at: datetime
    """When the domain was registered. Present when the registration resource exists."""

    domain_name: str
    """
    Provides a fully qualified domain name (FQDN), including the extension (e.g.,
    `example.com`, `mybrand.app`). The domain name uniquely identifies a
    registration. Cloudflare permits only one registration per domain, making the
    domain name a natural idempotency key for registration requests.
    """

    expires_at: Optional[datetime] = None
    """When the domain registration expires.

    Ready registrations include this value; only `registration_pending` may return
    null.
    """

    locked: bool
    """Whether the domain is locked for transfer."""

    privacy_mode: Literal["off", "redaction"]
    """Current WHOIS privacy mode for the registration."""

    status: Literal["active", "registration_pending", "expired", "suspended", "redemption_period", "pending_delete"]
    """Current registration status.

    - `active`: The domain operates with an active registration.
    - `registration_pending`: Registration remains in progress.
    - `expired`: The domain registration expired.
    - `suspended`: The registry suspended the domain.
    - `redemption_period`: The domain entered the redemption grace period.
    - `pending_delete`: The registry scheduled the domain for deletion.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ContentGetParams"]


class ContentGetParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    version_id: int
    """Specific version ID to retrieve. When omitted, the latest version is returned."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["RecipientDeleteResponse", "Resource"]


class Resource(BaseModel):
    error: str
    """Share Recipient error message."""

    resource_id: str
    """Share Resource identifier."""

    resource_version: int
    """Resource Version."""

    terminal: bool
    """Whether the error is terminal or will be continually retried."""


class RecipientDeleteResponse(BaseModel):
    """A recipient of a share.

    The `association_status` field tracks the
    lifecycle of the shared resources in the recipient account. All
    recipients are returned by the list endpoint regardless of status;
    filter client-side if only active recipients are needed.
    """

    id: str
    """Share Recipient identifier tag."""

    account_id: str
    """Account identifier."""

    association_status: Literal["associating", "associated", "disassociating", "disassociated"]
    """The current state of the recipient relative to the share.

    The `desired_association_status` (not exposed in the response) tracks the target
    state set by the API; the background reconciliation workflow drives
    `current_association_status` toward it.

    - `associating` — The recipient was recently added; the workflow is pushing
      shared resources into the recipient account.
    - `associated` — Shared resources have been successfully applied to the
      recipient account.
    - `disassociating` — The recipient was removed (via DELETE or PUT replacement);
      the workflow is removing shared resources from the recipient account.
    - `disassociated` — Shared resources have been removed from the recipient
      account. The recipient record remains in the database.
    """

    created: datetime
    """When the share was created."""

    modified: datetime
    """When the share was modified."""

    resources: Optional[List[Resource]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["SinkholeCreateParams"]


class SinkholeCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """An identifier for the resource."""

    name: Required[str]
    """The name of the sinkhole."""

    r2_bucket: str
    """The name of the R2 bucket to store results.

    Required if you want to store large request bodies in R2.
    """

    r2_id: str
    """The id of the R2 instance.

    Required if you want to store large request bodies in R2.
    """

    r2_secret: str
    """The secret key for the R2 API token.

    Required if you want to store large request bodies in R2.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["EnforcementRule"]


class EnforcementRule(BaseModel):
    expression: str
    """The filter expression that determines which requests the rule matches."""

    mode: Literal["min-friction", "max-security"]
    """The override mode Precursor applies to requests matching an enforcement rule.

    Unlike `default_mode`, this cannot be `off`.
    """

    id: Optional[str] = None
    """The read-only identifier that Cloudflare assigns to the rule."""

    description: Optional[str] = None
    """An informative description of the rule."""

    enabled: Optional[bool] = None
    """Whether the rule is active."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from .ingresses import (
    IngressesResource,
    AsyncIngressesResource,
    IngressesResourceWithRawResponse,
    AsyncIngressesResourceWithRawResponse,
    IngressesResourceWithStreamingResponse,
    AsyncIngressesResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ....types.intel import sinkhole_create_params, sinkhole_update_params
from ...._base_client import AsyncPaginator, make_request_options
from ....types.intel.sinkhole import Sinkhole

__all__ = ["SinkholesResource", "AsyncSinkholesResource"]


class SinkholesResource(SyncAPIResource):
    @cached_property
    def ingresses(self) -> IngressesResource:
        return IngressesResource(self._client)

    @cached_property
    def with_raw_response(self) -> SinkholesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SinkholesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SinkholesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SinkholesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        name: str,
        r2_bucket: str | Omit = omit,
        r2_id: str | Omit = omit,
        r2_secret: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Sinkhole]:
        """Create a new sinkhole.

        Logs of large request bodies will be truncated, but the
        full request body can be recorded in R2. If you wish to record large request
        bodies in R2, include the R2 key ID, key secret, and bucket name in the request
        body.

        Args:
          account_id: An identifier for the resource.

          name: The name of the sinkhole.

          r2_bucket: The name of the R2 bucket to store results. Required if you want to store large
              request bodies in R2.

          r2_id: The id of the R2 instance. Required if you want to store large request bodies in
              R2.

          r2_secret: The secret key for the R2 API token. Required if you want to store large request
              bodies in R2.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/intel/sinkholes", account_id=account_id),
            body=maybe_transform(
                {
                    "name": name,
                    "r2_bucket": r2_bucket,
                    "r2_id": r2_id,
                    "r2_secret": r2_secret,
                },
                sinkhole_create_params.SinkholeCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Sinkhole]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Sinkhole]], ResultWrapper[Sinkhole]),
        )

    def update(
        self,
        sinkhole_id: str,
        *,
        account_id: str,
        name: str,
        r2_bucket: str | Omit = omit,
        r2_id: str | Omit = omit,
        r2_secret: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """Replaces the name or R2 configuration of the specified sinkhole.

        This is a full
        replacement. All fields, including r2_secret, must be re-supplied. Omitting
        r2_secret overwrites the stored value with an empty string.

        Args:
          account_id: An identifier for the resource.

          name: The name of the sinkhole.

          r2_bucket: The name of the R2 bucket to store results. Required if you want to store large
              request bodies in R2.

          r2_id: The id of the R2 instance. Required if you want to store large request bodies in
              R2.

          r2_secret: The secret key for the R2 API token. Required if you want to store large request
              bodies in R2.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sinkhole_id:
            raise ValueError(f"Expected a non-empty value for `sinkhole_id` but received {sinkhole_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/intel/sinkholes/{sinkhole_id}", account_id=account_id, sinkhole_id=sinkhole_id
            ),
            body=maybe_transform(
                {
                    "name": name,
                    "r2_bucket": r2_bucket,
                    "r2_id": r2_id,
                    "r2_secret": r2_secret,
                },
                sinkhole_update_params.SinkholeUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[Sinkhole]:
        """
        Lists sinkholes owned by the account for redirecting malicious traffic.

        Args:
          account_id: An identifier for the resource.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/intel/sinkholes", account_id=account_id),
            page=SyncSinglePage[Sinkhole],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=Sinkhole,
        )

    def delete(
        self,
        sinkhole_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """Delete the specified sinkhole.

        The sinkhole must not have any active ingress
        rules defined. A 409 response code indicates that this condition is not met.

        Args:
          account_id: An identifier for the resource.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sinkhole_id:
            raise ValueError(f"Expected a non-empty value for `sinkhole_id` but received {sinkhole_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/intel/sinkholes/{sinkhole_id}", account_id=account_id, sinkhole_id=sinkhole_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def get(
        self,
        sinkhole_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Sinkhole]:
        """
        Get the specified sinkhole by its unique identifier.

        Args:
          account_id: An identifier for the resource.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sinkhole_id:
            raise ValueError(f"Expected a non-empty value for `sinkhole_id` but received {sinkhole_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/intel/sinkholes/{sinkhole_id}", account_id=account_id, sinkhole_id=sinkhole_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Sinkhole]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Sinkhole]], ResultWrapper[Sinkhole]),
        )


class AsyncSinkholesResource(AsyncAPIResource):
    @cached_property
    def ingresses(self) -> AsyncIngressesResource:
        return AsyncIngressesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSinkholesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSinkholesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSinkholesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSinkholesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        name: str,
        r2_bucket: str | Omit = omit,
        r2_id: str | Omit = omit,
        r2_secret: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Sinkhole]:
        """Create a new sinkhole.

        Logs of large request bodies will be truncated, but the
        full request body can be recorded in R2. If you wish to record large request
        bodies in R2, include the R2 key ID, key secret, and bucket name in the request
        body.

        Args:
          account_id: An identifier for the resource.

          name: The name of the sinkhole.

          r2_bucket: The name of the R2 bucket to store results. Required if you want to store large
              request bodies in R2.

          r2_id: The id of the R2 instance. Required if you want to store large request bodies in
              R2.

          r2_secret: The secret key for the R2 API token. Required if you want to store large request
              bodies in R2.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/intel/sinkholes", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "r2_bucket": r2_bucket,
                    "r2_id": r2_id,
                    "r2_secret": r2_secret,
                },
                sinkhole_create_params.SinkholeCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Sinkhole]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Sinkhole]], ResultWrapper[Sinkhole]),
        )

    async def update(
        self,
        sinkhole_id: str,
        *,
        account_id: str,
        name: str,
        r2_bucket: str | Omit = omit,
        r2_id: str | Omit = omit,
        r2_secret: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """Replaces the name or R2 configuration of the specified sinkhole.

        This is a full
        replacement. All fields, including r2_secret, must be re-supplied. Omitting
        r2_secret overwrites the stored value with an empty string.

        Args:
          account_id: An identifier for the resource.

          name: The name of the sinkhole.

          r2_bucket: The name of the R2 bucket to store results. Required if you want to store large
              request bodies in R2.

          r2_id: The id of the R2 instance. Required if you want to store large request bodies in
              R2.

          r2_secret: The secret key for the R2 API token. Required if you want to store large request
              bodies in R2.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sinkhole_id:
            raise ValueError(f"Expected a non-empty value for `sinkhole_id` but received {sinkhole_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/intel/sinkholes/{sinkhole_id}", account_id=account_id, sinkhole_id=sinkhole_id
            ),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "r2_bucket": r2_bucket,
                    "r2_id": r2_id,
                    "r2_secret": r2_secret,
                },
                sinkhole_update_params.SinkholeUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Sinkhole, AsyncSinglePage[Sinkhole]]:
        """
        Lists sinkholes owned by the account for redirecting malicious traffic.

        Args:
          account_id: An identifier for the resource.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/intel/sinkholes", account_id=account_id),
            page=AsyncSinglePage[Sinkhole],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=Sinkhole,
        )

    async def delete(
        self,
        sinkhole_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """Delete the specified sinkhole.

        The sinkhole must not have any active ingress
        rules defined. A 409 response code indicates that this condition is not met.

        Args:
          account_id: An identifier for the resource.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sinkhole_id:
            raise ValueError(f"Expected a non-empty value for `sinkhole_id` but received {sinkhole_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/intel/sinkholes/{sinkhole_id}", account_id=account_id, sinkhole_id=sinkhole_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def get(
        self,
        sinkhole_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Sinkhole]:
        """
        Get the specified sinkhole by its unique identifier.

        Args:
          account_id: An identifier for the resource.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sinkhole_id:
            raise ValueError(f"Expected a non-empty value for `sinkhole_id` but received {sinkhole_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/intel/sinkholes/{sinkhole_id}", account_id=account_id, sinkhole_id=sinkhole_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Sinkhole]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Sinkhole]], ResultWrapper[Sinkhole]),
        )


class SinkholesResourceWithRawResponse:
    def __init__(self, sinkholes: SinkholesResource) -> None:
        self._sinkholes = sinkholes

        self.create = to_raw_response_wrapper(
            sinkholes.create,
        )
        self.update = to_raw_response_wrapper(
            sinkholes.update,
        )
        self.list = to_raw_response_wrapper(
            sinkholes.list,
        )
        self.delete = to_raw_response_wrapper(
            sinkholes.delete,
        )
        self.get = to_raw_response_wrapper(
            sinkholes.get,
        )

    @cached_property
    def ingresses(self) -> IngressesResourceWithRawResponse:
        return IngressesResourceWithRawResponse(self._sinkholes.ingresses)


class AsyncSinkholesResourceWithRawResponse:
    def __init__(self, sinkholes: AsyncSinkholesResource) -> None:
        self._sinkholes = sinkholes

        self.create = async_to_raw_response_wrapper(
            sinkholes.create,
        )
        self.update = async_to_raw_response_wrapper(
            sinkholes.update,
        )
        self.list = async_to_raw_response_wrapper(
            sinkholes.list,
        )
        self.delete = async_to_raw_response_wrapper(
            sinkholes.delete,
        )
        self.get = async_to_raw_response_wrapper(
            sinkholes.get,
        )

    @cached_property
    def ingresses(self) -> AsyncIngressesResourceWithRawResponse:
        return AsyncIngressesResourceWithRawResponse(self._sinkholes.ingresses)


class SinkholesResourceWithStreamingResponse:
    def __init__(self, sinkholes: SinkholesResource) -> None:
        self._sinkholes = sinkholes

        self.create = to_streamed_response_wrapper(
            sinkholes.create,
        )
        self.update = to_streamed_response_wrapper(
            sinkholes.update,
        )
        self.list = to_streamed_response_wrapper(
            sinkholes.list,
        )
        self.delete = to_streamed_response_wrapper(
            sinkholes.delete,
        )
        self.get = to_streamed_response_wrapper(
            sinkholes.get,
        )

    @cached_property
    def ingresses(self) -> IngressesResourceWithStreamingResponse:
        return IngressesResourceWithStreamingResponse(self._sinkholes.ingresses)


class AsyncSinkholesResourceWithStreamingResponse:
    def __init__(self, sinkholes: AsyncSinkholesResource) -> None:
        self._sinkholes = sinkholes

        self.create = async_to_streamed_response_wrapper(
            sinkholes.create,
        )
        self.update = async_to_streamed_response_wrapper(
            sinkholes.update,
        )
        self.list = async_to_streamed_response_wrapper(
            sinkholes.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            sinkholes.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            sinkholes.get,
        )

    @cached_property
    def ingresses(self) -> AsyncIngressesResourceWithStreamingResponse:
        return AsyncIngressesResourceWithStreamingResponse(self._sinkholes.ingresses)
